"use strict";

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var PromisePool = require(".");

describe("PromisePool", function () {
  var promiseSpy = void 0;
  var promiseProducer = void 0;

  beforeEach(function () {
    promiseSpy = jest.fn();
    promiseProducer = function promiseProducer() {
      return new Promise(function (resolve, reject) {
        promiseSpy();
        resolve("success");
      });
    };
  });

  it("should await all promises", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var pool, results;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            pool = new PromisePool();

            pool.add(promiseProducer);
            pool.add(promiseProducer);
            _context.next = 5;
            return pool.all();

          case 5:
            results = _context.sent;

            expect(promiseSpy).toHaveBeenCalledTimes(2);
            expect(results.length).toBe(2);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it("should await all promises with no concurrency", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var pool, results;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            pool = new PromisePool({ concurrency: 1 });

            pool.add(promiseProducer);
            pool.add(promiseProducer);
            pool.add(promiseProducer);
            _context2.next = 6;
            return pool.all();

          case 6:
            results = _context2.sent;

            expect(promiseSpy).toHaveBeenCalledTimes(3);
            expect(results.length).toBe(3);

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it("should allow multiple concurrency", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var delayedPromiseSpy, delayedPromiseProducer, pool, results;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            delayedPromiseSpy = jest.fn();

            delayedPromiseProducer = function delayedPromiseProducer() {
              return new Promise(function (resolve, reject) {
                setTimeout(function () {
                  delayedPromiseSpy();
                  resolve("delayed");
                }, 1000);
              });
            };

            pool = new PromisePool({ concurrency: 2 });

            pool.add(delayedPromiseProducer);
            pool.add(promiseProducer);
            _context3.next = 7;
            return pool.all();

          case 7:
            results = _context3.sent;

            expect(delayedPromiseSpy).toBeCalled();
            expect(promiseSpy).toBeCalled();
            expect(results[0]).toBe("success");
            expect(results[1]).toBe("delayed");

          case 12:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it("can be called multiple times", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var pool, results;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            pool = new PromisePool({ concurrency: 2 });

            pool.add(promiseProducer);
            _context4.next = 4;
            return pool.all();

          case 4:
            pool.add(promiseProducer);
            pool.add(promiseProducer);
            _context4.next = 8;
            return pool.all();

          case 8:
            results = _context4.sent;

            expect(promiseSpy).toHaveBeenCalledTimes(3);
            expect(results.length).toBe(3);

          case 11:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  it("should throw if .all fails", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var pool, error, errorProducer;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            pool = new PromisePool({ concurrency: 2 });
            error = void 0;

            errorProducer = function errorProducer() {
              return new Promise(function (resolve, reject) {
                reject(new Error("Test"));
              });
            };

            _context5.prev = 3;

            pool.add(promiseProducer);
            pool.add(promiseProducer);
            pool.add(errorProducer);
            _context5.next = 9;
            return pool.all();

          case 9:
            _context5.next = 14;
            break;

          case 11:
            _context5.prev = 11;
            _context5.t0 = _context5["catch"](3);

            error = _context5.t0.message;

          case 14:
            expect(error).toBe("Test");

          case 15:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5, undefined, [[3, 11]]);
  })));

  it("should throw if any promise fails outside of .all", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    var pool, error, errorProducer;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            pool = new PromisePool({ concurrency: 2 });
            error = void 0;

            errorProducer = function errorProducer() {
              return new Promise(function (resolve, reject) {
                reject(new Error("Test"));
              });
            };

            pool.add(promiseProducer);
            pool.add(promiseProducer);
            pool.add(errorProducer);
            _context6.prev = 6;
            _context6.next = 9;
            return pool.all();

          case 9:
            _context6.next = 14;
            break;

          case 11:
            _context6.prev = 11;
            _context6.t0 = _context6["catch"](6);

            error = _context6.t0.message;

          case 14:
            expect(error).toBe("Test");

          case 15:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6, undefined, [[6, 11]]);
  })));
});